/*
 * Copyright 2007 TAKAI Naoto <takai@recompile.net>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

package net.recompile.webdb.comet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.concurrent.atomic.AtomicInteger;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.enterprise.web.connector.grizzly.comet.CometEvent;
import com.sun.enterprise.web.connector.grizzly.comet.CometHandler;

/**
 * O|[O`ɂCometHandlerB
 * 
 * @author TAKAI Naoto <takai@recompile.net>
 */
public class LongPollingCometHandler implements
		CometHandler<HttpServletResponse> {

	/** JE^[ւ̎Q */
	private final AtomicInteger counter;

	/** O */
	private final Logger logger = LoggerFactory
			.getLogger(LongPollingCometHandler.class);

	/** ֘AtꂽX|XIWFNg */
	private HttpServletResponse response;

	/**
	 * RXgN^B
	 * 
	 * @param counter
	 *            ֘AtJE^[IuWFNgB
	 */
	public LongPollingCometHandler(AtomicInteger counter) {
		this.counter = counter;
	}

	/**
	 * X|XIWFNg֘AtB
	 * 
	 * @param response
	 *            ֘At郌X|XIWFNgB
	 */
	public void attach(HttpServletResponse response) {
		logger.info("CometHandler({})ɃX|XIWFNg({})֘At܂B", this.toString(),
				response.toString());
		this.response = response;
	}

	/**
	 * CometContextʒmCxgMB
	 */
	public void onEvent(CometEvent event) throws IOException {
		logger.info("LongPollingCometHandler#onEvent(CometEvent)Ăяo܂B");
		if (CometEvent.NOTIFY == event.getType()) {
			logger.info("CometHandler({})Ɋ֘AtĂ郌X|XIWFNg({})"
					+ "o͂Aڑؒf܂B", this.toString(), response.toString());

			// wb_ɃJE^̒lݒ肷B
			Integer count = counter.get();
			response.addHeader("X-JSON", "{\"counter\":" + count + " }");

			// X|X{fBo͂B
			PrintWriter writer = response.getWriter();
			writer.write("success");
			writer.flush();

			// HTTPڑؒfB
			event.getCometContext().resumeCometHandler(this);
		}
	}

	public void onInitialize(CometEvent event) throws IOException {
		logger.info("LongPollingCometHandler#onInitialize(CometEvent)Ăяo܂B");
	}

	public void onInterrupt(CometEvent event) throws IOException {
		logger.info("LongPollingCometHandler#onInterrupt(CometEvent)Ăяo܂B");
		
		int count = counter.get();
		response.addHeader("X-JSON", "{\"counter\":" + count + " }");

		PrintWriter writer = response.getWriter();
		writer.write("success");
		writer.flush();
	}

	public void onTerminate(CometEvent event) throws IOException {
		logger.info("LongPollingCometHandler#onTerminate(CometEvent)Ăяo܂B");
	}

}